from .client import FuelCheckClient
from .dto import (
    AveragePrice, Variance, Station, Period, Price, FuelCheckError,
    GetFuelPricesResponse, FuelType, GetReferenceDataResponse,
    SortField, TrendPeriod
)

__all__ = ["FuelCheckClient", "AveragePrice", "Variance", "Station", "Period",
           "Price", "FuelCheckError", "GetFuelPricesResponse", "FuelType",
           "GetReferenceDataResponse", "SortField", "TrendPeriod"]
__version__ = "1.1.2"



from datetime import datetime
from enum import Enum
from typing import Optional, List, Any, Dict

from requests import Response


class Price(object):
    def __init__(self, fuel_type: str, price: float,
                 last_updated: Optional[datetime], price_unit: Optional[str],
                 station_code: Optional[int]) -> None:
        self.fuel_type = fuel_type
        self.price = price
        self.last_updated = last_updated
        self.price_unit = price_unit
        self.station_code = station_code

    @classmethod
    def deserialize(cls, data: Dict[str, Any]) -> 'Price':

        # Stupid API has two different date representations! :O
        lastupdated = None
        try:
            lastupdated = datetime.strptime(
                data['lastupdated'], '%d/%m/%Y %H:%M:%S')
        except ValueError:
            pass
        try:
            lastupdated = datetime.strptime(
                data['lastupdated'], '%Y-%m-%d %H:%M:%S')
        except ValueError:
            pass

        station_code = None # type: Optional[int]
        if 'stationcode' in data:
            station_code = int(data['stationcode'])

        return Price(
            fuel_type=data['fueltype'],
            price=data['price'],
            last_updated=lastupdated,
            price_unit=data.get('priceunit'),
            station_code=station_code
        )

    def __repr__(self) -> str:
        return '<Price fuel_type={} price={}>'.format(
            self.fuel_type, self.price)


class Station(object):
    def __init__(self, id: Optional[str], brand: str, code: int,
                 name: str, address: str) -> None:
        self.id = id
        self.brand = brand
        self.code = code
        self.name = name
        self.address = address

    @classmethod
    def deserialize(cls, data: Dict[str, Any]) -> 'Station':
        return Station(
            id=data.get('stationid'),
            brand=data['brand'],
            code=int(data['code']),
            name=data['name'],
            address=data['address']
        )

    def __repr__(self) -> str:
        return '<Station id={} code={} brand={} name={}>'.format(
            self.id, self.code, self.brand, self.name)


class Period(Enum):
    DAY = 'Day'
    MONTH = 'Month'
    YEAR = 'Year'
    WEEK = 'Week'


class Variance(object):
    def __init__(self, fuel_type: str, period: Period, price: float) -> None:
        self.fuel_type = fuel_type
        self.period = period
        self.price = price

    @classmethod
    def deserialize(cls, data: Dict[str, Any]) -> 'Variance':
        return Variance(
            fuel_type=data['Code'],
            period=Period(data['Period']),
            price=data['Price'],
        )

    def __repr__(self) -> str:
        return '<Variance fuel_type={} period={} price={}>'.format(
            self.fuel_type,
            self.period,
            self.price,
        )


class AveragePrice(object):
    def __init__(self, fuel_type: str, period: Period, price: float,
                 captured: datetime) -> None:
        self.fuel_type = fuel_type
        self.period = period
        self.price = price
        self.captured = captured

    @classmethod
    def deserialize(cls, data: Dict[str, Any]) -> 'AveragePrice':
        period = Period(data['Period'])

        captured_raw = data['Captured']
        if period in [Period.DAY, Period.WEEK, Period.MONTH]:
            captured = datetime.strptime(captured_raw, '%Y-%m-%d')
        elif period == Period.YEAR:
            captured = datetime.strptime(captured_raw, '%B %Y')
        else:
            captured = captured_raw

        return AveragePrice(
            fuel_type=data['Code'],
            period=period,
            price=data['Price'],
            captured=captured,
        )

    def __repr__(self) -> str:
        return ('<AveragePrice fuel_type={} period={} price={} '
                'captured={}>').format(
            self.fuel_type,
            self.period,
            self.price,
            self.captured
        )


class FuelType(object):
    def __init__(self, code: str, name: str) -> None:
        self.code = code
        self.name = name

    @classmethod
    def deserialize(cls, data: Dict[str, Any]) -> 'FuelType':
        return FuelType(
            code=data['code'],
            name=data['name']
        )


class TrendPeriod(object):
    def __init__(self, period: str, description: str) -> None:
        self.period = period
        self.description = description

    @classmethod
    def deserialize(cls, data: Dict[str, Any]) -> 'TrendPeriod':
        return TrendPeriod(
            period=data['period'],
            description=data['description']
        )


class SortField(object):
    def __init__(self, code: str, name: str) -> None:
        self.code = code
        self.name = name

    @classmethod
    def deserialize(cls, data: Dict[str, Any]) -> 'SortField':
        return SortField(
            code=data['code'],
            name=data['name']
        )


class GetReferenceDataResponse(object):
    def __init__(self, stations: List[Station], brands: List[str],
                 fuel_types: List[FuelType], trend_periods: List[TrendPeriod],
                 sort_fields: List[SortField]) -> None:
        self.stations = stations
        self.brands = brands
        self.fuel_types = fuel_types
        self.trend_periods = trend_periods
        self.sort_fields = sort_fields

    @classmethod
    def deserialize(cls, data: Dict[str, Any]) -> 'GetReferenceDataResponse':
        stations = [Station.deserialize(x) for x in data['stations']['items']]
        brands = [x['name'] for x in data['brands']['items']]
        fuel_types = [FuelType.deserialize(x) for x in
                      data['fueltypes']['items']]
        trend_periods = [TrendPeriod.deserialize(x) for x in
                         data['trendperiods']['items']]
        sort_fields = [SortField.deserialize(x) for x in
                       data['sortfields']['items']]

        return GetReferenceDataResponse(
            stations=stations,
            brands=brands,
            fuel_types=fuel_types,
            trend_periods=trend_periods,
            sort_fields=sort_fields
        )

    def __repr__(self) -> str:
        return ('<GetReferenceDataResponse stations=<{} stations>>').format(
            len(self.stations)
        )


class GetFuelPricesResponse(object):
    def __init__(self, stations: List[Station], prices: List[Price]) -> None:
        self.stations = stations
        self.prices = prices

    @classmethod
    def deserialize(cls, data: Dict[str, Any]) -> 'GetFuelPricesResponse':
        stations = [Station.deserialize(x) for x in data['stations']]
        prices = [Price.deserialize(x) for x in data['prices']]
        return GetFuelPricesResponse(
            stations=stations,
            prices=prices
        )


class FuelCheckError(Exception):
    def __init__(self, error_code: Optional[str] = None,
                 description: Optional[str] = None) -> None:
        super(FuelCheckError, self).__init__(description)
        self.error_code = error_code

    @classmethod
    def create(cls, response: Response) -> 'FuelCheckError':
        error_code = None
        description = response.text
        try:
            data = response.json()
            if 'errorDetails' in data:
                error_details = data['errorDetails']
                if type(error_details) == list and len(error_details) > 0:
                    error_details = error_details[0]
                    error_code = error_details.get('code')
                    description = error_details.get('description')
                elif type(error_details) == dict:
                    error_code = error_details.get('code')
                    description = error_details.get('message') # type: ignore

        except ValueError:
            pass

        return FuelCheckError(
            error_code=error_code,
            description=description
        )



import datetime
import requests
from typing import List, Optional, NamedTuple, Dict, Any

from .dto import (
    Price, Station, Variance, AveragePrice, FuelCheckError,
    GetReferenceDataResponse, GetFuelPricesResponse)

API_URL_BASE = 'https://api.onegov.nsw.gov.au/FuelCheckApp/v1/fuel'

PriceTrends = NamedTuple('PriceTrends', [
    ('variances', List[Variance]),
    ('average_prices', List[AveragePrice])
])

StationPrice = NamedTuple('StationPrice', [
    ('price', Price),
    ('station', Station)
])


class FuelCheckClient():
    def __init__(self, timeout: Optional[int] = 10) -> None:
        self._timeout = timeout

    def _format_dt(self, dt: datetime.datetime) -> str:
        return dt.strftime('%d/%m/%Y %H:%M:%S')

    def _get_headers(self) -> Dict[str, Any]:
        return {
            'requesttimestamp': self._format_dt(datetime.datetime.now())
        }

    def get_fuel_prices(self) -> GetFuelPricesResponse:
        """Fetches fuel prices for all stations."""
        response = requests.get(
            '{}/prices'.format(API_URL_BASE),
            headers=self._get_headers(),
            timeout=self._timeout,
        )

        if not response.ok:
            raise FuelCheckError.create(response)

        return GetFuelPricesResponse.deserialize(response.json())

    def get_fuel_prices_for_station(
            self,
            station: int
    ) -> List[Price]:
        """Gets the fuel prices for a specific fuel station."""
        response = requests.get(
            '{}/prices/station/{}'.format(API_URL_BASE, station),
            headers=self._get_headers(),
            timeout=self._timeout,
        )

        if not response.ok:
            raise FuelCheckError.create(response)

        data = response.json()
        return [Price.deserialize(data) for data in data['prices']]

    def get_fuel_prices_within_radius(
            self, latitude: float, longitude: float, radius: int,
            fuel_type: str, brands: Optional[List[str]] = None
    ) -> List[StationPrice]:
        """Gets all the fuel prices within the specified radius."""

        if brands is None:
            brands = []
        response = requests.post(
            '{}/prices/nearby'.format(API_URL_BASE),
            json={
                'fueltype': fuel_type,
                'latitude': latitude,
                'longitude': longitude,
                'radius': radius,
                'brand': brands,
            },
            headers=self._get_headers(),
            timeout=self._timeout,
        )

        if not response.ok:
            raise FuelCheckError.create(response)

        data = response.json()
        stations = {
            station['code']: Station.deserialize(station)
            for station in data['stations']
        }
        station_prices = []  # type: List[StationPrice]
        for serialized_price in data['prices']:
            price = Price.deserialize(serialized_price)
            station_prices.append(StationPrice(
                price=price,
                station=stations[price.station_code]
            ))

        return station_prices

    def get_fuel_price_trends(self, latitude: float, longitude: float,
                              fuel_types: List[str]) -> PriceTrends:
        """Gets the fuel price trends for the given location and fuel types."""
        response = requests.post(
            '{}/prices/trends/'.format(API_URL_BASE),
            json={
                'location': {
                    'latitude': latitude,
                    'longitude': longitude,
                },
                'fueltypes': [{'code': type} for type in fuel_types],
            },
            headers=self._get_headers(),
            timeout=self._timeout,
        )

        if not response.ok:
            raise FuelCheckError.create(response)

        data = response.json()
        return PriceTrends(
            variances=[
                Variance.deserialize(variance)
                for variance in data['Variances']
            ],
            average_prices=[
                AveragePrice.deserialize(avg_price)
                for avg_price in data['AveragePrices']
            ]
        )

    def get_reference_data(
            self,
            modified_since: Optional[datetime.datetime] = None
    ) -> GetReferenceDataResponse:
        """
        Fetches API reference data.

        :param modified_since: The response will be empty if no
        changes have been made to the reference data since this
        timestamp, otherwise all reference data will be returned.
        """

        if modified_since is None:
            modified_since = datetime.datetime(year=2010, month=1, day=1)

        response = requests.get(
            '{}/lovs'.format(API_URL_BASE),
            headers={
                'if-modified-since': self._format_dt(modified_since),
                **self._get_headers(),
            },
            timeout=self._timeout,
        )

        if not response.ok:
            raise FuelCheckError.create(response)

        # return response.text
        return GetReferenceDataResponse.deserialize(response.json())



import os
from setuptools import setup

readme_path = os.path.join(os.path.dirname(
    os.path.abspath(__file__)),
    'README.md',
)
long_description = open(readme_path).read()

setup(
    name='nsw-fuel-api-client',
    packages=['nsw_fuel'],
    author="Nick Whyte",
    author_email='nick@nickwhyte.com',
    description="API Client for NSW Government Fuel",
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/nickw444/nsw-fuel-api-client',
    zip_safe=False,
    install_requires=['requests'],
    classifiers=[
        'Intended Audience :: Developers',
        'Programming Language :: Python',
    ],
    test_suite="nsw_fuel_tests",
    tests_require=['requests-mock']
)
